import os
import re

from citrination_client.base import *
from citrination_client.search import *
from citrination_client.data import *
from citrination_client.models import *
from citrination_client.views.descriptors import *
from .client import CitrinationClient
from pkg_resources import get_distribution, DistributionNotFound

def __get_version():
    """
    Returns the version of this package, whether running from source or install

    :return: The version of this package
    """
    try:
        # Try local first, if missing setup.py, then use pkg info
        here = os.path.abspath(os.path.dirname(__file__))
        with open(os.path.join(here, "../setup.py")) as fp:
            version_file = fp.read()
            version_match = re.search(r"version=['\"]([^'\"]*)['\"]",
                                      version_file, re.M)
            if version_match:
                return version_match.group(1)
    except IOError:
        pass

    try:
        _dist = get_distribution('citrination_client')
        # Normalize case for Windows systems
        dist_loc = os.path.normcase(_dist.location)
        here = os.path.normcase(__file__)
        if not here.startswith(os.path.join(dist_loc, 'citrination_client')):
            # not installed, but there is another version that *is*
            raise DistributionNotFound
    except DistributionNotFound:
        raise RuntimeError("Unable to find version string.")
    else:
        return _dist.version

__version__ = __get_version()



from citrination_client.data.ingest import Ingester
from citrination_client.base import CitrinationClientError

class IngesterNotFoundError(CitrinationClientError):
    def __init__(self, message):
        super(message)

class IngesterList:
    """
    Class representation of a list of ingesters available for data upload on
    Citrination.

    :ivar ~.ingesters:
    :vartype ~.ingesters: list of :class:`Ingester`
    """

    def __init__(self, ingesters):
        """
        Constructor.

        :param ingesters: A list of ingesters
        :type ingesters: Union[list, :class:`IngesterList`]
        """
        if not isinstance(ingesters, list):
            raise TypeError('Expected ingesters to be of type list')
        else:
            self.ingesters = list(
                map(
                    lambda ingester: self._coerce_ingester(ingester), ingesters
                )
            )

    @property
    def ingester_count(self):
        """
        Returns the count of how many Ingesters are in the list

        :return: number of Ingesters in the list
        :rtype: int
        """
        return len(self.ingesters)

    def find_by_id(self, id):
        """
        Finds the ingester whose id matches the provided id

        :param id: The id of the desired ingester
        :type id: str

        :return: An ingester
        :rtype: :class:`Ingester`
        """
        try:
            # Find the first ingester whose id matches
            return next(
                filter(lambda ingester: ingester.id == id, self.ingesters)
            )
        except StopIteration:
            raise IngesterNotFoundError(
                'Unable to find ingester with id {}'.format(id)
            )

    def where(self, options):
        """
        Finds ingesters that match the search criteria

        :param options: contains string:string key value pairs, allowed keys
                        are those found in the Ingester.SEARCH_FIELDS constant
        :type options: dict

        :return: A list of matching ingesters
        :rtype: :class:`IngesterList`
        """
        self._validate_search_options(options)

        # If options is empty, just return a new copy of the IngesterList
        if not options:
            return IngesterList(self.ingesters)

        # Create a lambda that will check to see if all the values in the
        # dictionary are found in the corresponding attributes of the ingester
        #
        # i.e. If options = { "name": "apple", "description": "banana" }, the
        #      lambda will check that "apple" is in ingester.name and "banana"
        #      is in ingester.description - both conditions would need to be met
        #      in order for the lambda to return True
        match_lambda = lambda ingester: all(
            [self._ingester_contains(ingester, key, value) for key, value in options.items()]
        )

        # Get a list of ingesters that match the search options
        ingester_matches = list(filter(match_lambda, self.ingesters))

        return IngesterList(ingester_matches)

    def _ingester_contains(self, ingester, attribute, value):
        # Wrap in try except just in case one of the fields is None
        try:
            return value.lower() in getattr(ingester, attribute).lower()
        except Exception:
            return False

    def _coerce_ingester(self, ingester):
        """
        Helper method to ensure all members of the ingesters list are instances
        of Ingester

        :param ingester: either the dict or Ingester form of the ingester
        :type ingester: Union[dict, :class:`Ingester`]

        :return: an instance of Ingester
        :rtype :class:`Ingester`
        """
        # Return the original ingester if it is already an instance of Ingester
        return ingester if isinstance(ingester, Ingester) else Ingester(ingester)

    def _validate_search_options(self, options):
        if not isinstance(options, dict):
            raise TypeError(
                'Expected options to be of type dict, got {}'.format(
                    options.__class__
                )
            )

        # Check that the search keys are all valid
        [Ingester.validate_search_key(key, False) for key in options]

    def __str__(self):
        return "<IngesterList ingester_count={} ingesters={}>".format(
            self.ingester_count,
            list(map(lambda ingester: str(ingester), self.ingesters))
        )



from citrination_client.base import InvalidOptionError, CitrinationClientError
from copy import deepcopy

class ArgumentNotFoundError(CitrinationClientError):
    def __init__(self, message):
        super(message)

class Ingester:
    """
    Class representation of an ingester

    :ivar str ~.display_name:
    :ivar str ~.description:
    :ivar str ~.namespace:
    :ivar str ~.name:
    :ivar str ~.version:
    :ivar str ~.id:
    :ivar ~.arguments: any optional and/or required arguments for the ingester
    :vartype ~.arguments: list of dict
    """

    SEARCH_FIELDS = set([
        'description', 'display_name', 'name', 'namespace', 'version', 'id'
    ])
    """
    Attributes of an Ingester that can be searched when filtering through Ingesters
    via the IngestList#where method.
    """

    def __init__(self, ingester):
        """
        Constructor.

        :param ingester: An ingester
        :type ingester: dict
        """
        self._ingester = ingester;
        self.display_name = ingester['display_name']
        self.description = ingester['description']
        self.namespace = ingester['namespace']
        self.name = ingester['name']
        self.version = ingester['version']
        self.id = ingester['id']
        self.arguments = deepcopy(ingester['arguments']) or []

    @classmethod
    def validate_search_key(klass, key, suppress_error = True):
        """
        Returns whether or not a key correlates to a field that can be searched

        :param key: The key to check
        :type key: str
        :param suppress_error: When false, will raise an error for invalid keys
        :type suppress_error: bool
        :return: True if the key is valid, otherwise false
        :rtype: bool
        """
        key_present = key in klass.SEARCH_FIELDS

        if key_present or suppress_error:
            return key_present
        elif not key_present:
            raise InvalidOptionError(
                'Invalid key {}, accepted keys are {}'.format(
                    key, klass.SEARCH_FIELDS
                )
            )

    def clone(self):
        """
        Returns a copy of the ingester. Used by the IngestClient to apply values
        to a copy of the selected ingester before submission.

        :return: Returns a copy of the ingester based on a deep clone of the
                 _ingester dict that instantiated the instance.
        :return: class:`Ingester`
        """
        return self.__class__(deepcopy(self._ingester))

    def find_argument(self, name):
        """
        Finds an argument in self.arguments that matches the provided name.
        If there are no matches, raises an ArgumentNotFoundError

        :param name: The name of the argument
        :type name: str
        :return: The argument whose name matched
        :rtype: dict
        """
        try:
            # Find the first argument whose name matches
            return next(
                filter(lambda argument: argument['name'] == name, self.arguments)
            )
        except StopIteration:
            raise ArgumentNotFoundError(
                'Unable to find argument with name {}'.format(name)
            )

    def as_json(self):
        """
        Returns a dict that can used for ingest submission

        :return: A dict representation of the ingester
        :rtype: dict
        """
        return {
            "id": self.id,
            "name": self.name,
            "namespace": self.namespace,
            "version": self.version,
            "display_name": self.display_name,
            "description": self.description,
            "arguments": self.arguments
        }

    def __str__(self):
        return "<Ingester id='{}' display_name='{}' description='{}' num_arguments={}>".format(
            self.id, self.display_name, self.description, len(self.arguments)
        )



from .alloy_composition import AlloyCompositionColumn
from .categorical import CategoricalColumn
from .formulation import FormulationColumn
from .inorganic_chemical_formula import InorganicChemicalFormulaColumn
from .integer import IntColumn
from .organic_chemical_formula import OrganicChemicalFormulaColumn
from .real import RealColumn
from .vector import VectorColumn


class DescriptorConverter(object):

    @classmethod
    def convert(self, col_name, descriptor, role, units=None):
        column_type = descriptor['category']
        if column_type == CategoricalColumn.TYPE:
            categories = [str(v) for v in descriptor["descriptorValues"]]

            return CategoricalColumn(
                name=col_name,
                role=role,
                units=units,
                categories=categories
            )
        elif column_type == RealColumn.TYPE:
            lower_bound = descriptor["lowerBound"]
            upper_bound = descriptor["upperBound"]

            return RealColumn(
                name=col_name,
                role=role,
                units=units,
                lower_bound=lower_bound,
                upper_bound=upper_bound
            )
        elif column_type == IntColumn.TYPE:
            lower_bound = descriptor["lowerBound"]
            upper_bound = descriptor["upperBound"]

            return IntColumn(
                name=col_name,
                role=role,
                units=units,
                lower_bound=lower_bound,
                upper_bound=upper_bound
            )
        elif column_type == AlloyCompositionColumn.TYPE:
            balance_element = descriptor['balanceElement']
            basis = descriptor['basis']

            return AlloyCompositionColumn(
                name=col_name,
                role=role,
                units=units,
                balance_element=balance_element,
                basis=basis
            )
        elif column_type == InorganicChemicalFormulaColumn.TYPE:
            return InorganicChemicalFormulaColumn(
                name=col_name,
                role=role,
                units=units
            )
        elif column_type == OrganicChemicalFormulaColumn.TYPE:
            return OrganicChemicalFormulaColumn(
                name=col_name,
                role=role,
                units=units
            )
        elif column_type == FormulationColumn.TYPE:
            return FormulationColumn(
                name=col_name,
                role=role,
                units=units
            )
        elif column_type == VectorColumn.TYPE:
            length = descriptor["length"]

            return VectorColumn(
                name=col_name,
                role=role,
                units=units,
                length=length
            )



from citrination_client.models.design.constraints.base import BaseConstraint
from citrination_client.base.errors import CitrinationClientError
import citrination_client.util.maths as maths


class IntRangeConstraint(BaseConstraint):
    """
    Constrains an integer valued column to a range.
    """

    def __init__(self, name, minimum, maximum):
        """
        Constructor.

        :param name: The name of the column in the data
            view to which this constraint should be applied
        :type name: str
        :param minimum: The minimum allowed value
        :type minimum: int
        :param maximum: The maximum allowed value
        :type maximum: int
        """

        try:
            minimum_int = int(minimum)
            maximum_int = int(maximum)
        except TypeError:
            raise CitrinationClientError("IntRangeConstraint requires that minimum and maximum must be able to be cast to an integer")

        if minimum_int > maximum_int:
            raise CitrinationClientError("IntRangeConstraint requires that minimum be less than maximum")

        self._min = minimum_int
        self._max = maximum_int
        self._type = "integer"
        self._name = name

    def options(self):
        minimum = maths.convert_infinity_to_string(self._min)
        maximum = maths.convert_infinity_to_string(self._max)

        return {
            "min": minimum,
            "max": maximum
        }


from citrination_client.views.descriptors import CategoricalDescriptor
from citrination_client.views.descriptors import RealDescriptor
import re


class BaseDataViewBuilder(object):
    """
    A base class not intended to be instantiated directly.
    """

    def __init__(self, additional_config):
        self.configuration = dict(
            dataset_ids=[],
            group_by=[],
            model_type='default',
            descriptors=[]
        )
        self.configuration.update(additional_config)

    def dataset_ids(self, dataset_ids):
        """
        Sets the dataset ids to use for the view

        :param dataset_ids: Array of strings, one for each dataset id
        """
        self.configuration['dataset_ids'] = dataset_ids

    def model_type(self, model_type):
        """
        Sets the model type for the view

        :param model_type: A string of either linear (for a linear model), or default (for Citrine's default nonlinear model)
        """
        self.configuration['model_type'] = model_type

    def add_raw_descriptor(self, descriptor):
        """
        Add a raw descriptor dictionary object.

        :param descriptor: A descriptor as a dictionary
        """
        self.configuration['descriptors'].append(descriptor)

    def get_dagre_graph(self):
        """
        Function to generate a DAGRE graph based on the descriptors and relations added to the builder
        """
        nodes = []
        edges = []

        for desc in self.configuration['descriptors']:
            nodes.append({
                'label': desc['descriptor_key'],
                'attributes': {
                    'style': {
                        'fill': '#ff8200',
                        'stroke': '#453536'
                    }
                }
            })
        for relation in self.configuration['relations']:
            for input_key in relation['inputs']:
                edges.append({
                    "source": input_key,
                    "target": relation['output']
                })
        return {
            "nodes": nodes,
            "edges": edges
        }

    def _add_role_if_required(self, key, role):
        """
        Overridden by derived class if roles are being used

        :param key: descriptor key
        :param role: ignore, input, output, or latentVariable
        """
        # intentionally does nothing

    def add_formulation_descriptor(self, descriptor, dataview_client):
        """
        Add a formulation descriptor and automatically add ignored ingredient shares, component type and
        name categorical descriptors.

        :param descriptor: The formulation descriptor to add
        :param dataview_client: The dataview client, this is needed to obtain the ingredient share names
        """

        self.add_descriptor(descriptor)
        self._add_role_if_required(descriptor.key, "input")
        self.add_descriptor(CategoricalDescriptor("name", ["*"]))
        self._add_role_if_required("name", "ignore")
        self.add_descriptor(CategoricalDescriptor("component type", ["*"]))
        self._add_role_if_required("component type", "ignore")

        # Use template client to find the % share descriptors
        if self.configuration['dataset_ids']:
            client = dataview_client.search_template_client
            cols = client.get_available_columns(self.configuration['dataset_ids'])
            for col in cols:
                if re.match("% .* \(\w*, \w*\)", col):
                    self.add_descriptor(RealDescriptor(col, 0, 1))
                    self._add_role_if_required(col, "ignore")

    def build(self):
        """
        Returns a configuration object suitable for creating a data view.

        :return: See __init__ method for returned dict shape
        :rtype: :dict
        """
        return self.configuration



from citrination_client import BaseClient
from citrination_client.data.ingest import IngesterList
from citrination_client.data.ingest import Ingester

class IngestClient(BaseClient):
    """
    Data Views client.
    """

    def __init__(self, api_key, site="https://citrination.com", suppress_warnings=False, proxies=None):
        members = ["list_ingesters", "submit"]
        super(IngestClient, self).__init__(
            api_key, site, members, suppress_warnings, proxies
        )

    def list_ingesters(self):
        """
        Retrieves the list of available ingesters

        :return: The list of ingesters available for ingestion
        :rtype: :class:`IngesterList`
        """
        response = self._get_success_json(
            self._get(
                'v1/ingest/list_ingesters',
                failure_message='Failed to retrieve available ingesters'
            ),
        )['ingesters']

        return IngesterList(response)

    def submit(self, dataset_id, file_path, ingester, arguments = []):
        """
        Submits an ingest request using a custom (non-default) ingester

        :param dataset_id: The dataset being uploaded to
        :type dataset_id: str
        :param file_path: The destination path of the file
        :type file_path: str
        :param ingester: The ingester being used
        :type ingester: class:`citrination_client.data.ingest.Ingester`
        :param arguments: ingester arguments (optional), arguments should
                          contain keys "name" and "value"
        :type arguments: list of dict
        """
        ingester_with_values = self._apply_ingester_arguments(
            ingester, arguments
        )
        data = {
            "ingester": ingester_with_values.as_json(),
            "dataset_id": dataset_id,
            "target_path": file_path
        }

        self._get_success_json(
            self._post_json(
                'v1/ingest/submit',
                data,
                failure_message='Failed to submit ingestion request'
            )
        )

    def validate_ingester(self, ingester, arguments = []):
        """
        Validates that the ingester and argument values are valid.

        :param ingester: The ingester being used
        :type ingester: class:`citrination_client.data.ingest.Ingester`
        :param arguments: ingester arguments (optional), arguments should
                          contain keys "name" and "value"
        :type arguments: list of dict
        """
        ingester_with_values = self._apply_ingester_arguments(ingester, arguments)
        self._get_success_json(
            self._post_json(
                'v1/ingest/validate_ingester',
                { "ingester": ingester_with_values.as_json() },
                failure_message='Failed to submit ingestion request'
            )
        )

    def _apply_ingester_arguments(self, ingester, arguments = []):
        """
        Validates that the ingester and arguments passed to the #submit method
        are valid, and applies those arguments to a copy of the ingester.

        :param ingester: The ingester being used
        :type ingester: class:`citrination_client.data.ingest.Ingester`
        :param arguments: ingester arguments (optional), arguments should
                          contain keys "name" and "value"
        :type arguments: list of dict
        :return: a copy of the ingester with the argument values applied
        :rtype: citrination_client.data.ingest.Ingester
        """
        if not isinstance(ingester, Ingester):
            raise TypeError(
                "Expected ingester to be an instance of Ingester, received {}".format(
                    ingester.__class__
                )
            )
        else:
            self._validate_arguments_shape(arguments)

        ingester_copy = ingester.clone()

        for argument in arguments:
            ingester_copy.find_argument(argument['name'])['value'] = argument['value']

        return ingester_copy

    def _validate_arguments_shape(self, arguments):
        """
        Validates the shape of the arguments passed into _apply_ingester_arguments -
        that they are an array of dicts containing 'name' and 'value' keys

        :param arguments: ingester arguments (optional), arguments should
                          contain keys "name" and "value"
        :type arguments: list of dict
        """
        if not isinstance(arguments, list):
            raise TypeError(
                "Expected arguments to be an instance of list, received {}".format(
                    arguments.__class__
                )
            )
        else:
            improper_arguments = list(
                filter(
                    lambda arg: (
                        not isinstance(arg, dict) or
                        not 'name' in arg or
                        not 'value' in arg
                    ),
                    arguments
                )
            )
            if len(improper_arguments) > 0:
                raise TypeError(
                    "Expected all arguments to be dictionaries containing " \
                    "'name' and 'value' keys"
                )



from citrination_client.views.base_data_view_builder import BaseDataViewBuilder

MAX_USER_RELATIONS = 30

class AdvancedDataViewBuilder(BaseDataViewBuilder):
    """
    A more expressive interface for building data views with machine learning. Choose
    datasets, add descriptors, and specify relations, then call build() to
    return the configuration object needed by the data views API.

    Inherits from the BaseDataViewBuilder
    """

    def __init__(self):
        super(AdvancedDataViewBuilder, self).__init__(
            { 'builder': 'relation', 'relations': [] }
        )

    def add_descriptor(self, descriptor, group_by_key=False):
        """
        Add a descriptor column.

        :param descriptor: A Descriptor instance (e.g., RealDescriptor, InorganicDescriptor, etc.)
        :param group_by_key: Whether or not to group by this key during cross validation
        """
        descriptor.validate()

        self.configuration['descriptors'].append(descriptor.as_dict())

        if group_by_key:
            self.configuration["group_by"].append(descriptor.key)

    def add_relation(self, inputs, output, relation_type='lolo'):
        """
        Add a manual relation.  If no relations are manually added, Citrination will automatically generate
        relations when the view is created.

        :param inputs: Array of strings or a single string of descriptor key(s) for the relation inputs
        :param output: Single string descriptor key for the relation output
        :param relation_type: Kind of relation, currently only 'lolo' is supported
        """
        relation_obj = {}

        if isinstance(inputs, str):
            relation_obj['inputs'] = [inputs]
        elif isinstance(inputs, list):
            if len(inputs) == 0:
                raise ValueError("Inputs list must not be empty")
            relation_obj['inputs'] = inputs
        else:
            raise ValueError("Unexpected type for inputs, expecting either a string or list of strings")

        if isinstance(output, str):
            relation_obj['output'] = output
        else:
            raise ValueError("Unexpected type for output, expecting a string")

        if relation_type != 'lolo':
            raise ValueError("Currently, 'lolo' is the only allowed relation type")

        relation_obj['type'] = relation_type

        # check for duplicate
        relation_obj['inputs'].sort()
        existing_relations = self.configuration['relations']
        for ele in existing_relations:
            ele['inputs'].sort() # make sure they are sorted first
            if ele['inputs'] == relation_obj['inputs'] and \
                    ele['output'] == relation_obj['output']:
                raise ValueError("This relation duplicates an existing relation")

        # check limits
        if len(existing_relations) > MAX_USER_RELATIONS:
            raise ValueError("Maximum Relations Reached: Citrination only supports " + str(MAX_USER_RELATIONS) +
                             " user-defined relations. Please review the existing relations")

        # simplify descriptor testing
        descriptor_set = set()
        for desc in self.configuration['descriptors']:
            descriptor_set.add(desc['descriptor_key'])

        # check inputs exist
        for relation_input in relation_obj['inputs']:
            if relation_input not in descriptor_set:
                raise ValueError("Input " + relation_input + " is not defined as a descriptor")
            if relation_obj['inputs'].count(relation_input) > 1:
                raise ValueError("Input " + relation_input + " is listed more than once")
            if relation_input == output:
                raise ValueError("Output " + output + " is also an input")

        if output not in descriptor_set:
            raise ValueError("Output " + output + " is not defined as a descriptor")

        # check duplicate input

        self.configuration['relations'].append(relation_obj)



from .alloy_composition import AlloyCompositionColumn
from .base import BaseColumn
from .categorical import CategoricalColumn
from .descriptor_converter import DescriptorConverter
from .formulation import FormulationColumn
from .inorganic_chemical_formula import InorganicChemicalFormulaColumn
from .integer import IntColumn
from .organic_chemical_formula import OrganicChemicalFormulaColumn
from .real import RealColumn
from .vector import VectorColumn



from citrination_client.views.base_data_view_builder import BaseDataViewBuilder

class DataViewBuilder(BaseDataViewBuilder):
    """
    A low dimensional interface for building data views.  Choose datasets, add
    descriptors and other configuration options, then call build() to return the
    configuration object needed by the data views api

    Inherits from the BaseDataViewBuilder
    """

    def __init__(self):
        super(DataViewBuilder, self).__init__(
            { 'builder': 'simple', 'roles': dict() }
        )

    def add_descriptor(self, descriptor, role='ignore', group_by_key=False):
        """
        Add a descriptor column.

        :param descriptor: A Descriptor instance (e.g., RealDescriptor, InorganicDescriptor, etc.)
        :param role: Specify a role (input, output, latentVariable, or ignore)
        :param group_by_key: Whether or not to group by this key during cross validation
        """

        descriptor.validate()

        if descriptor.key in self.configuration["roles"]:
            raise ValueError("Cannot add a descriptor with the same name twice")

        self.configuration['descriptors'].append(descriptor.as_dict())
        self.configuration["roles"][descriptor.key] = role

        if group_by_key:
            self.configuration["group_by"].append(descriptor.key)

    def set_role(self, key, role):
        """
        Sets the role of a descriptor

        :param key: A descriptor key
        :type key: str
        :param role: (input, output, latentVariable, or ignore)
        :type role: str
        """
        self.configuration['roles'][key] = role

    def _add_role_if_required(self, key, role):
        """
        Overridden from base class.  The normal builder will set roles

        :param key: descriptor key
        :param role: ignore, input, output, or latentVariable
        """
        self.set_role(key, role)




from copy import deepcopy

class ModelReport(object):
    """
    An abstraction of a model report that wraps access to various sections
    of the report.
    """

    """
    :param raw_report: the dict representation of model report JSON
    :type: dict
    """
    def __init__(self, raw_report):
        self._raw_report = raw_report

    @property
    def model_name(self):
        """
        :return: the model output which this report corresponds to
        :rtype: str
        """
        return self._raw_report['model_name']

    @property
    def performance(self):
        """
        :return: dictionary of model performance metrics
        :rtype: dict
        """
        return self._raw_report['error_metrics']

    @property
    def feature_importances(self):
        """
        :return: list of key value pairs regarding feature importances
        :rtype: list of dict
        """
        return self._raw_report['feature_importances']

    @property
    def model_settings(self):
        """
        :return: dictionary of model settings
        :rtype: dict
        """
        return self._raw_report['model_settings']

    """
    WARNING - the dict format returned is unstable and may change over time.

    :return: a copy of the raw report that backs the instance.
    :rtype: dict
    """
    def as_json(self):
        return deepcopy(self._raw_report)

    def __str__(self):
        return "<ModelReport model_name='{}'>".format(
            self.model_name
        )



from .base import BaseConstraint
from .int_range import IntRangeConstraint
from .int_value import IntValueConstraint
from .real_value import RealValueConstraint
from .real_range import RealRangeConstraint
from .categorical import CategoricalConstraint
from .elemental_inclusion import ElementalInclusionConstraint
from .elemental_composition import ElementalCompositionConstraint



from citrination_client.views.client import DataViewsClient

from citrination_client.models import ModelsClient
from citrination_client.search import SearchClient
from citrination_client.data import DataClient
from citrination_client.util.credentials import get_preferred_credentials

class CitrinationClient(object):
    """
    The top level of the client hierarchy. Instantiating this class handles
    authentication information (api_key and site) and provides access to instances
    of each of the sub-clients, for more specific actions.

    Instantiation requires authentication information, but that can be provided
    via direct parameterization, environment variables, or a .citrination credentials
    file. See the tutorial on client Initialization for more information.
    """

    def __init__(self, api_key=None, site=None, suppress_warnings=False, proxies=None):
        """
        Constructor.

        :param api_key: Your API key for Citrination
        :type api_key: str
        :param site: The domain name of your Citrination deployment
            (the default is https://citrination.com)
        :type site: str
        :param suppress_warnings: A flag allowing you to suppress warning
            statements guarding against misuse printed to stdout.
        :type suppress_warnings: bool
        :param proxies: proxies to use when making HTTP requests. E.g.,
            proxies = { 'http': 'http://10.10.1.10:3128', 'https': 'http://10.10.1.10:1080' }
        :type proxies: dict(string, string)
        """
        api_key, site = get_preferred_credentials(api_key, site)
        self.models = ModelsClient(api_key, site, suppress_warnings=suppress_warnings, proxies=proxies)
        self.search = SearchClient(api_key, site, suppress_warnings=suppress_warnings, proxies=proxies)
        self.data = DataClient(api_key, site, suppress_warnings=suppress_warnings, proxies=proxies)
        self.data_views = DataViewsClient(api_key, site, suppress_warnings=suppress_warnings, proxies=proxies)

    def __repr__(self):
        return "['models', 'search', 'data', 'data_views']"
